@@ -97,6 +97,9 @@ struct _EomScrollViewPrivate {
 	GdkPixbuf *pixbuf;
 	cairo_surface_t *surface;
 
+	/* scale factor */
+	gint scale;
+
 	/* zoom mode, either ZOOM_MODE_FIT or ZOOM_MODE_FREE */
 	ZoomMode zoom_mode;
 
@@ -173,16 +176,10 @@ static cairo_surface_t *
 create_surface_from_pixbuf (EomScrollView *view, GdkPixbuf *pixbuf)
 {
 	cairo_surface_t *surface;
-	cairo_t *cr;
 
-	surface = gdk_window_create_similar_surface (gtk_widget_get_window (view->priv->display),
-							CAIRO_CONTENT_COLOR | CAIRO_CONTENT_ALPHA,
-							gdk_pixbuf_get_width (pixbuf),
-							gdk_pixbuf_get_height (pixbuf));
-	cr = cairo_create (surface);
-	gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
-	cairo_paint (cr);
-	cairo_destroy (cr);
+	surface = gdk_cairo_surface_create_from_pixbuf (pixbuf,
+			                                view->priv->scale,
+			                                gtk_widget_get_window (view->priv->display));
 
 	return surface;
 }
@@ -230,8 +227,8 @@ compute_scaled_size (EomScrollView *view, double zoom, int *width, int *height)
 	priv = view->priv;
 
 	if (priv->pixbuf) {
-		*width = floor (gdk_pixbuf_get_width (priv->pixbuf) * zoom + 0.5);
-		*height = floor (gdk_pixbuf_get_height (priv->pixbuf) * zoom + 0.5);
+		*width = floor (gdk_pixbuf_get_width (priv->pixbuf) / priv->scale * zoom + 0.5);
+		*height = floor (gdk_pixbuf_get_height (priv->pixbuf) / priv->scale * zoom + 0.5);
 	} else
 		*width = *height = 0;
 }
@@ -689,8 +686,8 @@ set_minimum_zoom_factor (EomScrollView *view)
 {
 	g_return_if_fail (EOM_IS_SCROLL_VIEW (view));
 
-	view->priv->min_zoom = MAX (1.0 / gdk_pixbuf_get_width (view->priv->pixbuf),
-				    MAX(1.0 / gdk_pixbuf_get_height (view->priv->pixbuf),
+	view->priv->min_zoom = MAX (1.0 / gdk_pixbuf_get_width (view->priv->pixbuf) / view->priv->scale,
+				    MAX(1.0 / gdk_pixbuf_get_height (view->priv->pixbuf) / view->priv->scale,
 					MIN_ZOOM_FACTOR) );
 	return;
 }
@@ -794,8 +791,8 @@ set_zoom_fit (EomScrollView *view)
 	gtk_widget_get_allocation (GTK_WIDGET(priv->display), &allocation);
 
 	new_zoom = zoom_fit_scale (allocation.width, allocation.height,
-				   gdk_pixbuf_get_width (priv->pixbuf),
-				   gdk_pixbuf_get_height (priv->pixbuf),
+				   gdk_pixbuf_get_width (priv->pixbuf) / priv->scale,
+				   gdk_pixbuf_get_height (priv->pixbuf) / priv->scale,
 				   priv->upscale);
 
 	if (new_zoom > MAX_ZOOM_FACTOR)
@@ -1295,7 +1292,8 @@ display_draw (GtkWidget *widget, cairo_t *cr, gpointer data)
 			cairo_matrix_init_translate (&translate, image_offset_x, image_offset_y);
 			cairo_matrix_multiply (&matrix, &matrix, &translate);
 		}
-		cairo_matrix_init_scale (&scale, priv->zoom, priv->zoom);
+		/* Zoom factor for SVGs is already scaled, so scale back to application pixels. */
+		cairo_matrix_init_scale (&scale, priv->zoom / priv->scale, priv->zoom / priv->scale);
 		cairo_matrix_multiply (&matrix, &matrix, &scale);
 		cairo_matrix_init_translate (&translate, xofs, yofs);
 		cairo_matrix_multiply (&matrix, &matrix, &translate);
@@ -1767,6 +1765,7 @@ eom_scroll_view_init (EomScrollView *view)
 	priv->display = g_object_new (GTK_TYPE_DRAWING_AREA,
 				      "can-focus", TRUE,
 				      NULL);
+	priv->scale = gtk_widget_get_scale_factor (GTK_WIDGET (priv->display));
 
 	gtk_widget_add_events (GTK_WIDGET (priv->display),
 			       GDK_EXPOSURE_MASK
@@ -2088,8 +2087,8 @@ view_on_drag_begin_cb (GtkWidget        *widget,
 	thumbnail = eom_image_get_thumbnail (image);
 
 	if  (thumbnail) {
-		width = gdk_pixbuf_get_width (thumbnail);
-		height = gdk_pixbuf_get_height (thumbnail);
+		width = gdk_pixbuf_get_width (thumbnail) / view->priv->scale;
+		height = gdk_pixbuf_get_height (thumbnail) / view->priv->scale;
 		gtk_drag_set_icon_pixbuf (context, thumbnail, width/2, height/2);
 		g_object_unref (thumbnail);
 	}
@@ -2162,7 +2161,7 @@ eom_scroll_view_set_popup (EomScrollView *view,
 	g_return_if_fail (EOM_IS_SCROLL_VIEW (view));
 	g_return_if_fail (view->priv->menu == NULL);
 
-	view->priv->menu = g_object_ref (menu);
+	view->priv->menu = g_object_ref (GTK_WIDGET (menu));
 
 	gtk_menu_attach_to_widget (GTK_MENU (view->priv->menu),
 				   GTK_WIDGET (view),
